#include "header.h"

GLsizei width = 512;
GLsizei height = 512;

int viewVolumeX = 4;
int viewVolumeY = 4;
int viewVolumeZ = 4;
float4 n, v;

float cornerToCenterSq;

int editingCornerIdx = -1;

float4 cubeCorner[8];
float4 cubeCornerProj[8];
float4x4 projMat;

bool isDebugMode = false;

void initGLSetting()
{
	glEnable(GL_DEPTH_TEST);
	initCube();
	float4 center = make_float4(0.f, 0.f, 0.f, 1.f);
	cornerToCenterSq = 3; // 1*1 + 1*1 + 1*1

	n = make_float4(0, 0, 1, 0); // x + y + z = 0
	v = n - center; v.w = 1;
}
void reshapeCB(int w, int h)
{
	width = w; height = h;
	glViewport(0, 0, width, height);
	glutPostRedisplay();
}
int findCornerIdx(float x, float y)
{
	int idx[] = {-1, -1};
	int find = 0;
	for (int i = 0; i < 8; i++)
	{
		float tx = cubeCorner[i].x - x;
		float ty = cubeCorner[i].y - y;
		if ((tx * tx + ty * ty) < 0.01)
		{
			idx[find] = i;
			find++;
		}
	}
	if (find == 0) return -1;
	else if (find == 1) return idx[0];
	else return (cubeCorner[idx[0]].z > cubeCorner[idx[1]].z) ? idx[0] : idx[1];
}
int ox, oy;
static inline float mousePosX(int x)
{
	return x / (float)width * viewVolumeX * 2 - viewVolumeX; // n = (0, 0, 1)
}
static inline float mousePosY(int y)
{
	return (height-y) / (float)height * viewVolumeY * 2 - viewVolumeY; // n = (0, 0, 1)
}
void mouseCB(int button, int state, int x, int y)
{
	if (GLUT_LEFT_BUTTON == button)
	{
		float _x, _y;
		if (state == GLUT_DOWN)
		{
			_x = mousePosX(x);
			_y = mousePosY(y);
			editingCornerIdx = findCornerIdx(_x, _y);
			ox = x; oy = y;
		}
		else editingCornerIdx = -1;
	}

	// TEST
	if ((editingCornerIdx != -1) && isDebugMode)
		printf("\n selected Idx: %d (%.3f, %.3f, %.3f)", editingCornerIdx, cubeCorner[editingCornerIdx].x,
						cubeCorner[editingCornerIdx].y, cubeCorner[editingCornerIdx].z);

	glutPostRedisplay();
}
void motionCB(int x, int y)
{
	if (editingCornerIdx != -1)
	{
		float nx = mousePosX(x);
		float ny = mousePosY(y);

		// get old point
		float3 oPoint = make_float3(cubeCorner[editingCornerIdx].x, 
									cubeCorner[editingCornerIdx].y, 
									cubeCorner[editingCornerIdx].z);
#if 1	/* Implementation 1 */
		// length(nPoint) < sqrt(3)
		if (nx*nx + ny*ny > cornerToCenterSq)
		{
			float inv = sqrtf(cornerToCenterSq / (nx*nx + ny*ny));
			nx *= inv; ny *= inv;
		}
		float3 delta = make_float3(nx-oPoint.x, ny-oPoint.y, 0);
		if (!EQ_ZERO(oPoint.z)) 
			delta.z = (-oPoint.x*delta.x - oPoint.y*delta.y)/oPoint.z;
		float deltaLen = sqrtf(dot(delta, delta));
		float oLen = sqrtf(dot(oPoint, oPoint));
		// get angle
		float angle; if (!EQ_ZERO(oLen)) angle = deltaLen / oLen;
		// get axis ( old x delta )
		float3 axis = cross(oPoint, delta);
		float aLen = sqrtf(dot(axis, axis));
		if (!EQ_ZERO(aLen)) axis = make_float3(axis.x/aLen, axis.y/aLen, axis.z/aLen);
		else return;

#else	/* Implementation 2 */
		// get new point
		float3 nPoint = make_float3(nx, ny, 0);
		nPoint.z = 3 - nx*nx - ny*ny;
		if (nPoint.z <= 0) nPoint.z = 0; else nPoint.z = sqrtf(nPoint.z);
		// normalize
		float oLen = sqrtf(dot(oPoint, oPoint));
		if (!EQ_ZERO(oLen)) oPoint = make_float3(oPoint.x/oLen, oPoint.y/oLen, oPoint.z/oLen);
		else return;
		float nLen = sqrtf(dot(nPoint, nPoint));
		if (!EQ_ZERO(nLen)) nPoint = make_float3(nPoint.x/nLen, nPoint.y/nLen, nPoint.z/nLen);
		else return;

		// get axis ( old x new )
		float3 axis = cross(oPoint, nPoint);
		float aLen = sqrtf(dot(axis, axis));
		if (!EQ_ZERO(aLen)) axis = make_float3(axis.x/aLen, axis.y/aLen, axis.z/aLen);
		else return;
		// get angle ( dot product )
		float dotON = dot(oPoint, nPoint); float angle;
		if ((dotON > 1) || (dotON < -1)) return; else angle = acosf(dotON);
#endif
		// get Quaternion
		Quaternion q = convertQuat(angle, axis.x, axis.y, axis.z);

		float4x4 rotMat;
		QuatToRotM(q, rotMat);
		for (int i = 0; i< 8; i++) cubeCorner[i] = mul(rotMat, cubeCorner[i]);

		ox = x; oy = y;

		// TEST
		if (isDebugMode)
		{
			printf("\n axis: (%.3f, %.3f, %.3f)", axis.x, axis.y, axis.z);
			printf("\n q: %.3f (%.3f, %.3f, %.3f)", q.w, q.x, q.y, q.z);
		}
	}	
	glutPostRedisplay();
}

void displayCB()
{
	glClearColor(1.f, 1.f, 1.f, 1.f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-viewVolumeX, viewVolumeX, -viewVolumeY, viewVolumeY, -viewVolumeZ, viewVolumeZ);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(v.x, v.y, v.z, 0, 0, 0, 0, 1, 0);

	draw3DAxis();
	
	drawCube();

	glutSwapBuffers();
}

void keyboardCB(unsigned char key, int x, int y)
{
	switch(key)
	{
	case 'i': case 'I':
		initCube();
		break;
	case 'c': case 'C':
		printf("\ncorner point");
		for (int i = 0; i < 8; i++)
			printf("\n [%d] %.4f %.4f %.4f", i, cubeCorner[i].x, cubeCorner[i].y, cubeCorner[i].z);
		break;
	case 'd': case 'D':
		isDebugMode ^= true;
		if (isDebugMode)	printf("\n\n[Debug mode on]\n");
		else				printf("\n\n[Debug mode off]\n");
		break;
	case (27): exit(0); break;
	default:
		break;
	}
	glutPostRedisplay();
}

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitWindowPosition(0, 0);
	glutInitWindowSize(width, height);
	glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB|GLUT_DEPTH);
	glutCreateWindow("OpenGL test");

	glutDisplayFunc(displayCB);
	glutReshapeFunc(reshapeCB);
	glutMouseFunc(mouseCB);
	glutMotionFunc(motionCB);
	glutKeyboardFunc(keyboardCB);

	initGLSetting();

	glutMainLoop();
	return 0;
}
