#include <math.h>
#include <GL/glut.h>
#include <iostream>

#define NUMLINES 7

double lines[NUMLINES][4] = {
#if 1
	{ -0.8, 0.8, -0.3, 0.8 }, 
	{ -0.8, -0.1, -0.2, 0.6 }, 
	{ -0.4, -0.4, -0.05, -0.35 }, 
	{ -0.05, 0.3, 0.25, -0.1 }, 
	{ 0.1, 0.85, 0.2, 0.4 }, 
	{ 0.6, 0.7, 0.8, 0.4 }, 
	{ 0.4, -0.7, 0.7, 0.05 }
#else
	{ 0, 0, 0.3, 0 }, { 0.2, -0.1, 0.3, 0.1 }, 	...
#endif
};
double colors[NUMLINES][3] = {
	{ 0.0, 0.0, 1.0 }, 
	{ 0.0, 1.0, 0.0 }, 
	{ 0.0, 1.0, 1.0 }, 
	{ 1.0, 0.0, 0.0 }, 
	{ 1.0, 0.0, 1.0 }, 
	{ 1.0, 1.0, 0.0 }, 
	{ 0.5, 0.5, 0.5 } 
};
void init() {
	glClearColor(0.0, 0.0, 0.0, 1.0);

	// Projection Matrix
	glMatrixMode(GL_PROJECTION); // glMatrixMode(_______(1)_______);
	glLoadIdentity();
	glOrtho(-1, 1, -1, 1, -1, 1);

	// ModelView Matrix
	glMatrixMode(GL_MODELVIEW);	// glMatrixMode(_______(2)_______);
	glLoadIdentity();

	// Depth Test
	glEnable(GL_DEPTH_TEST); // glEnable(_______(3)_______);
}
// .
void drawline(int index) {
	glColor3d(0.0, 0.0, 0.0);
	glBegin(GL_LINES); // glBegin(_______(4)_______);
	glVertex3d(lines[index][0], lines[index][1], 0.1);
	glVertex3d(lines[index][2], lines[index][3], 0.1);
	glEnd();
}

void get_normal(int index, double normal[2]) {
	double len;
	normal[0] = lines[index][1] - lines[index][3];
	normal[1] = lines[index][2] - lines[index][0];
	len = sqrt(normal[0] * normal[0] + normal[1] * normal[1]);
	normal[0] /= len;
	normal[1] /= len;
}
// Voronoi
void drawvoronoi(int index)
{
	double normal[2];
	const double kSize = 2.0;

	double _num6 = -kSize; // ADDED: _______(6)_______

	get_normal(index, normal);
	glColor3dv(colors[index]);
	glPushMatrix();	// _______(5)_______();
	glTranslated(lines[index][0], lines[index][1], _num6); // _______(6)_______);
	glutSolidCone(kSize, kSize, 64, 1);
	glPopMatrix();	//_______(7)_______();
	//
	glPushMatrix();	// _______(5)_______();
	glTranslated(lines[index][2], lines[index][3], _num6); // _______(6)_______);
	glutSolidCone(kSize, kSize, 64, 1);
	glPopMatrix();	//_______(7)_______();

	//
	glBegin(GL_QUADS); // glBegin(_____(8_)____);
	glVertex3d(lines[index][0], lines[index][1], 0);
	glVertex3d(lines[index][0] + normal[0] * kSize,
		lines[index][1] + normal[1] * kSize, _num6); // _______(6)_______);
	glVertex3d(lines[index][2] + normal[0] * kSize,
		lines[index][3] + normal[1] * kSize, _num6); // _______(6)_______);
	glVertex3d(lines[index][2], lines[index][3], 0);

	glVertex3d(lines[index][0], lines[index][1], 0);
	glVertex3d(lines[index][0] - normal[0] * kSize,
		lines[index][1] - normal[1] * kSize, _num6); // _______(6)_______);
	glVertex3d(lines[index][2] - normal[0] * kSize,
		lines[index][3] - normal[1] * kSize, _num6); // _______(6)_______);
	glVertex3d(lines[index][2], lines[index][3], 0);
	glEnd();
}
void display() {
	// Depth Buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	// glClear(_______(9)_______ | _______(10)_______);
	int index = 1;
	glColor3d(0.0, 0.0, 0.0);
	glBegin(GL_LINES);
	glVertex3d(lines[index][0], lines[index][1], 0.1);
	glVertex3d(lines[index][2], lines[index][3], 0.1);
	glEnd();
	//
	for (int i=0; i<NUMLINES; ++i)
		drawvoronoi(i);
	//
	for (int i=0; i<NUMLINES; ++i)
		drawline(i);
	glutSwapBuffers();
}
void main(int argc, char **argv) 
{
	glutInit(&argc, argv);
	glutInitWindowSize(512, 512);
	glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB|GLUT_DEPTH);
	glutCreateWindow("OpenGL test");

//	...

	init();
	glutDisplayFunc(display);

	glutMainLoop();
}